@@ -14,9 +14,9 @@
 #include <string.h>
 
 #include "dict-common/dict-common.h"
+#include "dict-common/dict-internals.h"
 #include "dict-common/dict-utils.h"     // patch_subscript
 #include "dict-common/idiom.h"
-#include "dict-file/read-dict.h"        // dict_error2
 #include "string-id.h"
 #include "string-set.h"
 
@@ -24,12 +24,16 @@
 
 /* ======================================================================== */
 
-static void free_dict_node_recursive(Dict_node * dn)
+void free_dict_node_recursive(Dict_node * dn)
 {
-	if (dn == NULL) return;
-	free_dict_node_recursive(dn->left);
-	free_dict_node_recursive(dn->right);
-	free(dn);
+	while (dn != NULL)
+	{
+		free_dict_node_recursive(dn->left);
+
+		Dict_node * rn = dn->right;
+		free(dn);
+		dn = rn;
+	}
 }
 
 void free_dictionary_root(Dictionary dict)
@@ -40,10 +44,6 @@ void free_dictionary_root(Dictionary dict)
 	dict->Exp_pool = NULL;
 }
 
-void dict_node_noop(Dictionary dict)
-{
-}
-
 /* ======================================================================== */
 /**
  * Dictionary entry comparison and ordering functions.
@@ -69,7 +69,7 @@ void dict_node_noop(Dictionary dict)
 /**
  * dict_order_strict - order two dictionary words in proper sort order.
  * Return zero if the strings match, else return in a unique order.
- * The order is NOT (locale-dependent) UTF8 sort order; its ordered
+ * The order is NOT (locale-dependent) UTF8 sort order; it's ordered
  * based on numeric values of single bytes.  This will uniquely order
  * UTF8 strings, just not in a LANG-dependent (locale-dependent) order.
  * But we don't need/want locale-dependent ordering!
@@ -169,71 +169,8 @@ static inline int dict_order_wild(const char * s, const Dict_node * dn)
 }
 #undef D_DOW
 
-Dict_node * dict_node_new(void)
-{
-	return (Dict_node*) malloc(sizeof(Dict_node));
-}
-
 /* ======================================================================== */
-#if 0
-/**
- * dict_match --  return true if strings match, else false.
- * A "bare" string (one without a subscript) will match any corresponding
- * string with a subscript; so, for example, "make" and "make.n" are
- * a match.  If both strings have subscripts, then the subscripts must match.
- *
- * A subscript is the part that follows the SUBSCRIPT_MARK.
- */
-static bool dict_match(const char * s, const char * t)
-{
-	while ((*s == *t) && (*s != '\0')) { s++; t++; }
 
-	if (*s == *t) return true; /* both are '\0' */
-	if ((*s == 0) && (*t == SUBSCRIPT_MARK)) return true;
-	if ((*s == SUBSCRIPT_MARK) && (*t == 0)) return true;
-
-	return false;
-}
-
-/**
- * prune_lookup_list -- discard all list entries that don't match string
- * Walk the lookup list (of right links), discarding all nodes that do
- * not match the dictionary string s. The matching is dictionary matching:
- * subscripted entries will match "bare" entries.
- */
-static Dict_node * prune_lookup_list(Dict_node * restrict llist, const char * restrict s)
-{
-	Dict_node *dn, *dnx, *list_new;
-
-	list_new = NULL;
-	for (dn = llist; dn != NULL; dn = dnx)
-	{
-		dnx = dn->right;
-		/* now put dn onto the answer list, or free it */
-		if (dict_match(dn->string, s))
-		{
-			dn->right = list_new;
-			list_new = dn;
-		}
-		else
-		{
-			free(dn);
-		}
-	}
-
-	/* now reverse the list back */
-	llist = NULL;
-	for (dn = list_new; dn != NULL; dn = dnx)
-	{
-		dnx = dn->right;
-		dn->right = llist;
-		llist = dn;
-	}
-	return llist;
-}
-#endif
-
-/* ======================================================================== */
 static bool subscr_match(const char *s, const Dict_node * dn)
 {
 	const char * s_sub = get_word_subscript(s);
@@ -263,29 +200,43 @@ rdictionary_lookup(Dict_node * restrict llist,
                    bool boolean_lookup,
                    int (*dict_order)(const char *, const Dict_node *))
 {
-	int m;
-	Dict_node * dn_new;
 	if (dn == NULL) return llist;
 
-	m = dict_order(s, dn);
+	int m = dict_order(s, dn);
 
-	if (m >= 0)
+	// The tree is in alphabetical order. Move down either the
+	// left or right branch, until a string match is found.
+	while (m != 0)
 	{
-		llist = rdictionary_lookup(llist, dn->right, s, boolean_lookup, dict_order);
+		if (m > 0)
+			dn = dn->right;
+		else if (m < 0)
+			dn = dn->left;
+
+		if (dn == NULL) return llist;
+
+		m = dict_order(s, dn);
 	}
-	if ((m == 0) && (dict_order != dict_order_wild || subscr_match(s, dn)))
+
+	// There might be more than one perfect match. Recurse to get it.
+	if (dn->right)
+		llist = rdictionary_lookup(llist, dn->right, s, boolean_lookup, dict_order);
+
+	if (dict_order != dict_order_wild || subscr_match(s, dn))
 	{
+		dn->use_count++;
 		if (boolean_lookup) return dn;
-		dn_new = dict_node_new();
+		Dict_node * dn_new = dict_node_new();
 		*dn_new = *dn;
 		dn_new->right = llist;
 		dn_new->left = dn; /* Currently only used for inserting idioms */
 		llist = dn_new;
 	}
-	if (m <= 0)
-	{
+
+	// There might be more than one perfect match. Recurse to get it.
+	if (dn->left)
 		llist = rdictionary_lookup(llist, dn->left, s, boolean_lookup, dict_order);
-	}
+
 	return llist;
 }
 
@@ -309,20 +260,22 @@ bool dict_node_exists_lookup(Dictionary dict, const char *s)
 	return !!rdictionary_lookup(NULL, dict->root, s, true, dict_order_bare);
 }
 
-void dict_node_free_list(Dict_node *llist)
+/**
+ * strict_lookup_list() - return exact match in the dictionary
+ *
+ * Returns a pointer to a lookup list of the words in the dictionary.
+ *
+ * This list is made up of Dict_nodes, linked by their right pointers.
+ * The node, file and string fields are copied from the dictionary.
+ *
+ * The list normally has 0 or 1 elements, unless the given word
+ * appears more than once in the dictionary.
+ *
+ * The returned list must be freed with dict_node_free_lookup().
+ */
+Dict_node * strict_lookup_list(const Dictionary dict, const char *s)
 {
-	Dict_node * n;
-	while (llist != NULL)
-	{
-		n = llist->right;
-		free(llist);
-		llist = n;
-	}
-}
-
-void dict_node_free_lookup(Dictionary dict, Dict_node *llist)
-{
-	dict_node_free_list(llist);
+	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_strict);
 }
 
 /**
@@ -347,44 +300,56 @@ Dict_node * dict_node_wild_lookup(Dictionary dict, const char *s)
 	return result;
 }
 
-#if 0
-/**
- * abridged_lookup_list() - return lookup list of words in the dictionary
- *
- * Returns a pointer to a lookup list of the words in the dictionary.
- * Excludes any idioms that contain the word; use
- * dictionary_lookup_list() to obtain the complete list.
- *
- * This list is made up of Dict_nodes, linked by their right pointers.
- * The node, file and string fields are copied from the dictionary.
- *
- * The returned list must be freed with dict_node_free_lookup().
- */
-static Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
-{
-	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
-}
-#endif
-
-/**
- * strict_lookup_list() - return exact match in the dictionary
- *
- * Returns a pointer to a lookup list of the words in the dictionary.
- *
- * This list is made up of Dict_nodes, linked by their right pointers.
- * The node, file and string fields are copied from the dictionary.
- *
- * The list normally has 0 or 1 elements, unless the given word
- * appears more than once in the dictionary.
- *
- * The returned list must be freed with dict_node_free_lookup().
- */
-Dict_node * strict_lookup_list(const Dictionary dict, const char *s)
-{
-	return rdictionary_lookup(NULL, dict->root, s, false, dict_order_strict);
-}
-
 /* ======================================================================== */
+/*
+ * "Expressions" -- these encode the binary-tree structure of the
+ * hand-written dictionaries. These are binary trees, with leaves of
+ * connectors joined up by the binary operators `&`, `or`.
+ *
+ * The in-RAM representation is NOT a binary tree; instead it is a tree
+ * of lists. An Exp node contains a tag: `AND_type`, `OR_type`, etc.
+ * The `operand_next` field is used to hold a linked list which is
+ * joined up with the given Exp type. This is more efficient to traverse,
+ * and also saves space, as compared to an ordinary binary tree.
+ *
+ * Example:  (A or B or C or D) becomes
+ *
+ *   or
+ *    \
+ *     A -> B -> C -> D
+ *
+ * The slash \ is the `operand_first` field. The horizontal arrows are
+ * the `operand_next` field. Here's a more complicated example:
+ *
+ *    (A or (E & F) or C or D)
+ *
+ * becomes
+ *
+ *   or
+ *    \
+ *     A -> and -> C -> D
+ *            \
+ *             E -> F
+ *
+ * This structure has performance implications: the `build_disjuncts()`
+ * code will traverse the `operand_next` fields in a loop, but will make
+ * a recursive call for each `operand_first` (thus buying a stack frame.)
+ * The original example (A or B or C or D) could have been written as
+ * (A or (B or (C or D))) and so
+ *
+ *   or
+ *    \
+ *     A -> or
+ *           \
+ *            B -> or
+ *                  \
+ *                   C -> D
+ *
+ * But this would be a bad idea, since (1) more Exp nodes are required,
+ * and (2) the recursive calls in `build_disjuncts()` really do add up
+ * for large expressions.
+ */
+
 /**
  * Allocate a new Exp node.
  */
@@ -440,7 +405,7 @@ Exp *make_unary_node(Pool_desc *mp, Exp * e)
  * Create an expression that joins together `nl` and `nr`.
  * The join type can be either `AND_type` or `OR_type`.
  */
-Exp * make_join_node(Pool_desc *mp, Exp_type t, Exp* nl, Exp* nr)
+Exp * make_join_node(Pool_desc *mp, Exp* nl, Exp* nr, Exp_type t)
 {
 	Exp* n;
 
@@ -460,7 +425,7 @@ Exp * make_join_node(Pool_desc *mp, Exp_type t, Exp* nl, Exp* nr)
  */
 Exp * make_and_node(Pool_desc *mp, Exp* nl, Exp* nr)
 {
-	return make_join_node(mp, AND_type, nl, nr);
+	return make_join_node(mp, nl, nr, AND_type);
 }
 
 /**
@@ -469,7 +434,7 @@ Exp * make_and_node(Pool_desc *mp, Exp* nl, Exp* nr)
  */
 Exp * make_or_node(Pool_desc *mp, Exp* nl, Exp* nr)
 {
-	return make_join_node(mp, OR_type, nl, nr);
+	return make_join_node(mp, nl, nr, OR_type);
 }
 
 Exp * make_connector_node(Dictionary dict, Pool_desc *mp,
@@ -625,7 +590,6 @@ static int dup_word_status(Dictionary dict, const Dict_node *newnode)
 
 static bool dup_word_error(Dictionary dict, Dict_node *newnode)
 {
-
 	if (dup_word_status(dict, newnode) == 1) return false;
 
 	if (dict->allow_duplicate_words == 0)
@@ -640,8 +604,19 @@ static bool dup_word_error(Dictionary dict, Dict_node *newnode)
 		if (dup_word_status(dict, newnode) == 1) return false;
 	}
 
-	dict_error2(dict, "Ignoring word which has been multiply defined:",
-	            newnode->string);
+
+	if (IS_DYNAMIC_DICT(dict))
+	{
+		prt_error("Error: While handling storage-node\n  \"%s\":\n"
+		          "Ignoring word which has been multiply defined: \"%s\"\n",
+		          dict->name, newnode->string);
+	} else {
+		// File-backed dictionary sets a line number, for debugging.
+		prt_error("Error: While parsing dictionary \"%s\":\n"
+		          "Ignoring word which has been multiply defined: \"%s\"\n"
+		          "\t Line %d\n",
+		          dict->name, newnode->string, dict->line_number);
+	}
 
 	/* Too late to skip insertion - insert it with a null expression. */
 	newnode->exp = make_zeroary_node(dict->Exp_pool);
