@@ -312,10 +312,12 @@ static int linkage_equiv_p(Linkage lpv, Linkage lnx)
 		Link * plk = &lpv->link_array[li];
 		Link * nlk = &lnx->link_array[li];
 
-		// String set guarantees that if the pointer differs,
-		// then the string does too.
-		if (plk->link_name != nlk->link_name)
-			return strcmp(plk->link_name, nlk->link_name);
+		// Note (see intersect_strings()):
+		// link_name is not always in the same string set, so inequality
+		// test cannot be done here.
+		if (plk->link_name == nlk->link_name) continue;
+		int lncmp = strcmp(plk->link_name, nlk->link_name);
+		if (lncmp) return lncmp;
 	}
 
 	// Compare words. The chosen_disjuncts->word_string is the
@@ -338,8 +340,13 @@ static int linkage_equiv_p(Linkage lpv, Linkage lnx)
 			if (NULL == ndj) continue;
 			return 1;
 		}
-		if (pdj->word_string != ndj->word_string)
-			return strcmp(pdj->word_string, ndj->word_string);
+
+		// Note (see build_word_expressions()):
+		// word_string is not always in the same string set, so inequality
+		// test cannot be done here.
+		if (pdj->word_string == ndj->word_string) continue;
+		int wscmp = strcmp(pdj->word_string, ndj->word_string);
+		if (wscmp) return wscmp;
 	}
 
 	// Compare connector types at the link endpoints. If we are here,
@@ -347,7 +354,7 @@ static int linkage_equiv_p(Linkage lpv, Linkage lnx)
 	// names were the same. The connector types might still differ,
 	// due to intersection. The multi-connector flag might differ.
 	// However, neither of these are likely. It is plausible to skip
-	// this check entirely, its mostly a CPU-time-waster that will
+	// this check entirely, it's mostly a CPU-time-waster that will
 	// never find any differences for the almost any situation.
 	for (uint32_t li=0; li<lpv->num_links; li++)
 	{
@@ -357,7 +364,7 @@ static int linkage_equiv_p(Linkage lpv, Linkage lnx)
 		if (plk->lc != nlk->lc)
 		{
 			if (plk->lc->desc != nlk->lc->desc)
-				return strcmp(plk->lc->desc->string, nlk->lc->desc->string);
+				return strcmp(connector_string(plk->lc), connector_string(nlk->lc));
 
 			int md = plk->lc->multi - nlk->lc->multi;
 			if (md) return md;
@@ -365,7 +372,7 @@ static int linkage_equiv_p(Linkage lpv, Linkage lnx)
 		if (plk->rc != nlk->rc)
 		{
 			if (plk->rc->desc != nlk->rc->desc)
-				return strcmp(plk->rc->desc->string, nlk->rc->desc->string);
+				return strcmp(connector_string(plk->rc), connector_string(nlk->rc));
 
 			int md = plk->rc->multi - nlk->rc->multi;
 			if (md) return md;
@@ -453,7 +460,7 @@ static void deduplicate_linkages(Sentence sent, int linkage_limit)
 	    !sent->overflowed && (sent->num_linkages_found <= linkage_limit)))
 		return;
 
-	// Deduplicate the valid linkages only; its not worth wasting
+	// Deduplicate the valid linkages only; it's not worth wasting
 	// CPU time on the rest.  Sorting guarantees that the valid
 	// linkages come first.
 	uint32_t nl = sent->num_valid_linkages;
