@@ -44,7 +44,7 @@
 #define PRx(x) fprintf(stderr, ""#x)
 #define PR(...) true
 
-/* Indicator that this connector cannot be used -- that its "obsolete".  */
+/* Indicator that this connector cannot be used -- that it's "obsolete".  */
 #define BAD_WORD (MAX_SENTENCE+1)
 
 typedef uint8_t WordIdx_m;     /* Storage representation of word index */
@@ -185,8 +185,8 @@ GNUC_UNUSED static void print_power_table(Sentence sent, power_table *pt)
  *     nearer and nearer words.
  *
  *  2) Two deep connectors cannot attach to each other; i.e. a deep
- *     connector can only attache to a shallow one, and a shallow
- *     connector can attache to any connector.
+ *     connector can only attach to a shallow one, and a shallow
+ *     connector can attach to any connector.
  *     (A connectors is deep if it is not the first in its list; it
  *     is shallow if it is the first in its list; it is deepest if it
  *     is the last on its list.)
@@ -276,7 +276,7 @@ static void put_into_power_table(Pool_desc *mp, unsigned int size, C_list **t,
 {
 	C_list **e = get_power_table_entry(size, t, c);
 
-	assert(NULL != e, "put_into_power_table: Overflow");
+	assert(NULL != e, "Overflow");
 	assert(c->refcount > 0, "refcount %d", c->refcount);
 
 	C_list *m = pool_alloc(mp);
@@ -447,11 +447,15 @@ static void power_table_init(Sentence sent, Tracon_sharing *ts, power_table *pt)
 static void clean_table(unsigned int size, C_list **t)
 {
 	/* Table entry tombstone. */
-#define UC_NUM_TOMBSTONE ((connector_hash_t)-1)
-	static condesc_t desc_no_match =
+#define UC_NUM_TOMBSTONE ((connector_uc_hash_t)-1)
+	static condesc_more_t cm_no_match =
 	{
 		.string = "TOMBSTONE",
+	};
+	static condesc_t desc_no_match =
+	{
 		.uc_num = UC_NUM_TOMBSTONE, /* get_power_table_entry() will skip. */
+		.more = &cm_no_match
 	};
 	static Connector con_no_match =
 	{
@@ -466,7 +470,7 @@ static void clean_table(unsigned int size, C_list **t)
 
 		while (NULL != *m)
 		{
-			assert(0 <= (*m)->c->refcount, "clean_table: refcount < 0 (%d)",
+			assert(0 <= (*m)->c->refcount, "refcount < 0 (%d)",
 			       (*m)->c->refcount);
 			if (0 == (*m)->c->refcount)
 			{
@@ -1308,8 +1312,8 @@ static void cms_table_delete(multiset_table *mt)
 static unsigned int cms_hash(const char *s)
 {
 	unsigned int i = 5381;
-	if (islower((int) *s)) s++; /* skip head-dependent indicator */
-	while (isupper((int) *s))
+	if (islower((unsigned int)*s)) s++; /* skip head-dependent indicator */
+	while (is_connector_name_char(*s))
 	{
 		i = ((i << 5) + i) + *s;
 		s++;
@@ -1317,9 +1321,9 @@ static unsigned int cms_hash(const char *s)
 	return (i & (CMS_SIZE-1));
 }
 
-static void reset_last_criterion(multiset_table *cmt, const char *ctiterion)
+static void reset_last_criterion(multiset_table *cmt, const char *criterion)
 {
-	unsigned int h = cms_hash(ctiterion);
+	unsigned int h = cms_hash(criterion);
 
 	for (Cms *cms = cmt->cms_table[h]; cms != NULL; cms = cms->next)
 		cms->last_criterion = false;
@@ -1346,14 +1350,14 @@ static void reset_last_criterion(multiset_table *cmt, const char *ctiterion)
  */
 static bool can_form_link(const char *s, const char *t, const char *e)
 {
-	if (islower(*t)) t++; /* Skip head-dependent indicator */
-	while (isupper(*s))
+	if (islower((unsigned char)*t)) t++; /* Skip head-dependent indicator */
+	while (is_connector_name_char(*s))
 	{
 		if (*s != *t) return false;
 		s++;
 		t++;
 	}
-	if (isupper(*t)) return false;
+	if (is_connector_name_char(*t)) return false;
 	while (*t != '\0')
 	{
 		if (*s == '\0') return true;
@@ -1492,7 +1496,7 @@ static bool all_connectors_exist(multiset_table *cmt, const char *pp_link)
 	ppdebug("check PP-link=%s\n", pp_link);
 
 	const char *s;
-	for (s = pp_link; isupper(*s); s++) {}
+	for (s = pp_link; is_connector_name_char(*s); s++) {}
 
 	/* The first iteration of the next loop handles the special case
 	 * which occurs if there were 0 subscripts. */
@@ -2187,7 +2191,7 @@ unsigned int pp_and_power_prune(Sentence sent, Tracon_sharing *ts,
 	if ((num_deleted > 0) && !no_mlink)
 	{
 		pc.ml = build_mlink_table(sent, ml);
-		print_time(opts, "Built_mlink_table%s", pc.ml ? "" : " (empty)");
+		print_time(opts, "Built mlink_table%s", pc.ml ? "" : " (empty)");
 		if (pc.ml != NULL)
 		{
 			if (null_count == 0)
@@ -2204,7 +2208,7 @@ unsigned int pp_and_power_prune(Sentence sent, Tracon_sharing *ts,
 		if ((num_deleted > 0) && !no_mlink)
 		{
 			pc.ml = build_mlink_table(sent, ml);
-			print_time(opts, "Built_mlink_table%s", pc.ml ? "" : " (empty)");
+			print_time(opts, "Built mlink_table%s", pc.ml ? "" : " (empty)");
 			if (pc.ml != NULL)
 			{
 				if (null_count == 0)
