@@ -13,13 +13,12 @@
 
 #include "api-structures.h"
 #include "connectors.h"
-#include "corpus/corpus.h"
-#include "dict-common/dict-utils.h"   // For size_of_expression()
+#include "dict-common/dict-utils.h"     // size_of_expression
 #include "disjunct-utils.h"
 #include "linkage/linkage.h"
-#include "post-process/post-process.h" // for compute_domain_names()
+#include "post-process/post-process.h"  // compute_domain_names
 #include "print.h"
-#include "tokenize/word-structures.h" // For Word_struct
+#include "tokenize/word-structures.h"   // Word_struct
 #include "wcwidth.h"
 
 #define LEFT_WALL_SUPPRESS ("Wd") /* If this connector is used on the wall, */
@@ -44,7 +43,7 @@
  * FIXME Long link names between more distant words may still not
  * fit the space between these words.
  *
- * Return the number of bytes needed for the all the words, including
+ * Return the number of bytes needed for all the words, including
  * the space needed for the link names as described above.  Note that
  * this byte length might be less than the glyph width! e.g. the
  * ASCII chars in the range 01 to 1F usually print in two columns,
@@ -66,7 +65,7 @@ set_centers(const Linkage linkage, int center[], int word_offset[],
 	{
 		Link *l = &linkage->link_array[n];
 
-		if ((l->lw + 1 == l->rw) && (NULL != l->link_name))
+		if (l->lw + 1 == l->rw)
 		{
 			link_len[l->rw] = strlen(l->link_name) +
 				(DEPT_CHR == connector_string(l->rc)[0]) +
@@ -215,15 +214,13 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 	longest = 0;
 	for (link=0; link<N_links; link++)
 	{
-		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
-		assert (linkage_get_link_lword(linkage, link) < SIZE_MAX);
+		assert (linkage_get_link_lword(linkage, link) < SIZE_MAX, "Missing word in link");
 		if (linkage_get_link_num_domains(linkage, link) > longest)
 			longest = linkage_get_link_num_domains(linkage, link);
 	}
 	for (link=0; link<N_links; link++)
 	{
-		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
-		assert (linkage_get_link_lword(linkage, link) < SIZE_MAX);
+		assert (linkage_get_link_lword(linkage, link) < SIZE_MAX, "Missing word in link");
 		dname = linkage_get_link_domain_names(linkage, link);
 		for (j=0; j<linkage_get_link_num_domains(linkage, link); ++j) {
 			append_string(s, " (%s)", dname[j]);
@@ -243,45 +240,8 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 	return dyn_str_take(s);
 }
 
-char * linkage_print_senses(Linkage linkage)
-{
-	dyn_str * s = dyn_str_new();
-#ifdef USE_CORPUS
-	Linkage_info *lifo = linkage->info;
-	Sense *sns;
-	size_t nwords;
-	WordIdx w;
-
-	lg_corpus_linkage_senses(linkage);
-
-	nwords = linkage->num_words;
-	for (w=0; w<nwords; w++)
-	{
-		sns = lg_get_word_sense(linkage, w);
-		while (sns)
-		{
-			int idx = lg_sense_get_index(sns);
-			const char * wd = lg_sense_get_subscripted_word(sns);
-			const char * dj = lg_sense_get_disjunct(sns);
-			const char * sense = lg_sense_get_sense(sns);
-			double score = lg_sense_get_score(sns);
-			append_string(s, "%d %s dj=%s sense=%s score=%f\n",
-			              idx, wd, dj, sense, score);
-			sns = lg_sense_next(sns);
-		}
-	}
-
-#else
-	dyn_strcat(s, "Corpus statistics is not enabled in this version\n");
-#endif
-	return dyn_str_take(s);
-}
-
 char * linkage_print_disjuncts(const Linkage linkage)
 {
-#ifdef USE_CORPUS
-	double score;
-#endif
 	const char * dj;
 	int w;
 	dyn_str * s = dyn_str_new();
@@ -291,7 +251,7 @@ char * linkage_print_disjuncts(const Linkage linkage)
 	for (w = 0; w < nwords; w++)
 	{
 		int pad = 21;
-		double cost;
+		float cost;
 		const char *infword;
 		Disjunct *disj = linkage->chosen_disjuncts[w];
 		if (NULL == disj) continue;
@@ -309,14 +269,10 @@ char * linkage_print_disjuncts(const Linkage linkage)
 
 		dj = linkage_get_disjunct_str(linkage, w);
 		if (NULL == dj) dj = "";
-		cost = linkage_get_disjunct_cost(linkage, w);
+		cost = (float)linkage_get_disjunct_cost(linkage, w);
 
-#ifdef USE_CORPUS
-		score = linkage_get_disjunct_corpus_score(linkage, w);
-		append_string(s, "%*s    %5.3f %6.3f %s\n", pad, infword, cost, score, dj);
-#else
-		append_string(s, "%*s    %5.3f  %s\n", pad, infword, cost, dj);
-#endif
+		append_string(s, "%*s    %6s  %s\n",
+		              pad, infword, cost_stringify(cost), dj);
 	}
 	return dyn_str_take(s);
 }
@@ -339,82 +295,79 @@ static char *
 build_linkage_postscript_string(const Linkage linkage,
                                 bool display_walls, ps_ctxt_t *pctx)
 {
-	int link, i,j;
-	int d;
-	bool print_word_0, print_word_N;
+	bool print_word_0 = true, print_word_N = true;
 	int N_links = linkage->num_links;
 	Link *ppla = linkage->link_array;
-	dyn_str * string;
-	int N_words_to_print;
 
-	string = dyn_str_new();
-
-	if (!display_walls) {
+	/* Do we want to print the left and right walls? */
+	if (!display_walls)
+	{
 		int N_wall_connectors = 0;
-		bool suppressor_used = false;
-		for (j=0; j<N_links; j++) {
-			if (ppla[j].lw == 0) {
+		for (int j=0; j<N_links; j++)
+		{
+			if (0 == ppla[j].lw)
+			{
 				if (ppla[j].rw == linkage->num_words-1) continue;
-				N_wall_connectors ++;
-				if (easy_match(connector_string(ppla[j].lc), LEFT_WALL_SUPPRESS)) {
-					suppressor_used = true;
+				if (easy_match(connector_string(ppla[j].lc), LEFT_WALL_SUPPRESS))
+					print_word_0 = false;
+
+				if (++N_wall_connectors > 1)
+				{
+					print_word_0 = true;
+					break;
 				}
 			}
 		}
-		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors != 1));
-	}
-	else print_word_0 = true;
 
-	if (!display_walls) {
-		int N_wall_connectors = 0;
-		bool suppressor_used = false;
-		for (j=0; j<N_links; j++) {
-			if (ppla[j].rw == linkage->num_words-1) {
-				N_wall_connectors ++;
-				if (easy_match(connector_string(ppla[j].lc), RIGHT_WALL_SUPPRESS)) {
-					suppressor_used = true;
+		N_wall_connectors = 0;
+		for (int j=0; j<N_links; j++)
+		{
+			if (ppla[j].rw == linkage->num_words-1)
+			{
+				if (easy_match(connector_string(ppla[j].lc), RIGHT_WALL_SUPPRESS))
+					print_word_N = false;
+
+				if (++N_wall_connectors > 1)
+				{
+					print_word_N = true;
+					break;
 				}
 			}
 		}
-		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors != 1));
 	}
-	else print_word_N = true;
 
-	if (print_word_0) d=0; else d=1;
-
-	i = 0;
-	N_words_to_print = linkage->num_words;
+	int N_words_to_print = linkage->num_words;
 	if (!print_word_N) N_words_to_print--;
 
+	/* The postscriipt string */
+	dyn_str * string = dyn_str_new();
 	dyn_strcat(string, "[");
-	for (j=d; j<N_words_to_print; j++) {
-		if ((i%10 == 0) && (i>0)) dyn_strcat(string, "\n");
-		i++;
+
+	int doff = print_word_0 ? 0 : 1;
+	for (int j=doff; j<N_words_to_print; j++)
+	{
+		if ((j%10 == 0) && (j>doff)) dyn_strcat(string, "\n");
 		append_string(string, "(%s)", linkage->word[j]);
 	}
 	dyn_strcat(string,"]");
 	dyn_strcat(string,"\n");
 
 	dyn_strcat(string,"[");
-	j = 0;
-	for (link=0; link<N_links; link++) {
+	int k = 0;
+	for (int link=0; link<N_links; link++) {
 		if (!print_word_0 && (ppla[link].lw == 0)) continue;
 		if (!print_word_N && (ppla[link].rw == linkage->num_words-1)) continue;
-		// if (ppla[link]->lw == SIZE_MAX) continue;
-		assert (ppla[link].lw != SIZE_MAX);
-		if ((j%7 == 0) && (j>0)) dyn_strcat(string,"\n");
-		j++;
-		append_string(string,"[%zu %zu %d",
-				ppla[link].lw - d, ppla[link].rw - d,
+		if ((k%7 == 0) && (k>0)) dyn_strcat(string,"\n");
+		k++;
+		append_string(string,"[%d %d %d",
+				ppla[link].lw - doff, ppla[link].rw - doff,
 				pctx->link_heights[link]);
 		append_string(string," (%s)]", ppla[link].link_name);
 	}
 	dyn_strcat(string,"]");
 	dyn_strcat(string,"\n");
 	dyn_strcat(string,"[");
-	for (j=0; j < pctx->N_rows; j++ )
+	for (int j=0; j < pctx->N_rows; j++)
 	{
 		if (j>0) append_string(string, " %d", pctx->row_starts[j]);
 		else append_string(string,"%d", pctx->row_starts[j]);
@@ -430,8 +383,8 @@ static void diagram_alloc_tmpmem(size_t **start, char ***pic, char ***xpic,
                                  size_t *cur_height, size_t max_height,
                                  size_t max_bytes, size_t num_cols)
 {
-	assert(num_cols < max_bytes);
-	assert(max_height > *cur_height);
+	assert(num_cols <= max_bytes, "Columns overflow");
+	assert(max_height > *cur_height, "New diagram height is too small");
 
 	*start = realloc(*start, max_height * sizeof(size_t));
 	*pic = realloc(*pic, max_height * sizeof(char *));
@@ -452,6 +405,43 @@ static void diagram_alloc_tmpmem(size_t **start, char ***pic, char ***xpic,
 	*cur_height = max_height;
 }
 
+typedef struct
+{
+	Link *lnk;
+	unsigned int len;          /* Length of this link */
+} link_by_length;
+
+static int by_link_len(const void *a, const void *b)
+{
+	const link_by_length * const *la = a;
+	const link_by_length * const *lb = b;
+
+	return (*la)->len - (*lb)->len;
+}
+
+static void sort_link_lengths(Link *ppla, link_by_length *ll,
+                              unsigned int N_links)
+{
+	link_by_length *ll_tmp = alloca(N_links * sizeof(*ll_tmp));
+	link_by_length **lla = alloca(N_links * sizeof(*lla));
+
+	for (unsigned int j = 0; j < N_links; j++)
+	{
+		Link *lnk = &ppla[j];
+		assert(lnk->link_name != NULL, "Missing link name");
+
+		lla[j] = &ll_tmp[j];
+		ll_tmp[j].lnk = lnk;
+		ll_tmp[j].len = (unsigned int)(lnk->rw - lnk->lw);
+	}
+
+	qsort(lla, N_links, sizeof(*lla), by_link_len);
+
+	/* Return the sort result through the ll parameter. */
+	for (unsigned int j = 0; j < N_links; j++)
+		ll[j] = *lla[j];
+}
+
 static void diagram_free_tmpmem(size_t *start, char **pic, char **xpic,
                                 size_t cur_height)
 {
@@ -490,62 +480,58 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	unsigned int i, j, k, cl, cr, inc, row, top_row, top_row_p1;
 	const char *s;
 	char *t;
-	bool print_word_0 , print_word_N;
+	bool print_word_0 = true, print_word_N = true;
 	int *center = alloca((linkage->num_words+1)*sizeof(int));
 	int *word_offset = alloca((linkage->num_words+1) * sizeof(*word_offset));
-	unsigned int link_length;
 	unsigned int N_links = linkage->num_links;
 	Link *ppla = linkage->link_array;
 	dyn_str * string;
 	unsigned int N_words_to_print;
 
-	// Avoid pathological case and the resulting crash.
+	// Avoid a pathological case and the resulting crash.
 	if (0 == linkage->num_words) return strdup("");
 
+	link_by_length *ll = alloca(N_links * sizeof(*ll));
+	sort_link_lengths(ppla, ll, N_links);
+
 	string = dyn_str_new();
 
-	/* Do we want to print the left wall? */
+	/* Do we want to print the left and right walls? */
 	if (!display_walls)
 	{
 		int N_wall_connectors = 0;
-		bool suppressor_used = false;
 		for (j=0; j<N_links; j++)
 		{
 			if (0 == ppla[j].lw)
 			{
 				if (ppla[j].rw == linkage->num_words-1) continue;
-				N_wall_connectors ++;
 				if (easy_match(connector_string(ppla[j].lc), LEFT_WALL_SUPPRESS))
+					print_word_0 = false;
+
+				if (++N_wall_connectors > 1)
 				{
-					suppressor_used = true;
+					print_word_0 = true;
+					break;
 				}
 			}
 		}
-		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
-					|| (N_wall_connectors != 1));
-	}
-	else print_word_0 = true;
 
-	/* Do we want to print the right wall? */
-	if (!display_walls)
-	{
-		int N_wall_connectors = 0;
-		bool suppressor_used = false;
+		N_wall_connectors = 0;
 		for (j=0; j<N_links; j++)
 		{
 			if (ppla[j].rw == linkage->num_words-1)
 			{
-				N_wall_connectors ++;
 				if (easy_match(connector_string(ppla[j].lc), RIGHT_WALL_SUPPRESS))
+					print_word_N = false;
+
+				if (++N_wall_connectors > 1)
 				{
-					suppressor_used = true;
+					print_word_N = true;
+					break;
 				}
 			}
 		}
-		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
-					|| (N_wall_connectors != 1));
 	}
-	else print_word_N = true;
 
 	N_words_to_print = linkage->num_words;
 	if (!print_word_N) N_words_to_print--;
@@ -574,88 +560,78 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	top_row = 0;
 
 	// Longer links are printed above the lower links.
-	for (link_length = 1; link_length < N_words_to_print; link_length++)
+	for (j=0; j<N_links; j++)
 	{
-		for (j=0; j<N_links; j++)
+		Link *lnk = ll[j].lnk;
+
+		if (!print_word_0 && (lnk->lw == 0)) continue;
+		/* Gets rid of the irrelevant link to the left wall */
+		if (!print_word_N && (lnk->rw == linkage->num_words-1)) continue;
+
+		/* Put it into the lowest position */
+		/* Keep in mind that cl, cr are "columns" not "bytes" */
+		cl = center[lnk->lw];
+		cr = center[lnk->rw];
+		for (row=0; row < max_height; row++)
 		{
-			assert (ppla[j].lw != SIZE_MAX);
-			if (NULL == ppla[j].link_name) continue;
-			if (((unsigned int) (ppla[j].rw - ppla[j].lw)) != link_length)
-			  continue;
-			if (!print_word_0 && (ppla[j].lw == 0)) continue;
-			/* Gets rid of the irrelevant link to the left wall */
-			if (!print_word_N && (ppla[j].rw == linkage->num_words-1)) continue;
-
-			/* Put it into the lowest position */
-			/* Keep in mind that cl, cr are "columns" not "bytes" */
-			cl = center[ppla[j].lw];
-			cr = center[ppla[j].rw];
-			for (row=0; row < max_height; row++)
+			for (k=cl+1; k<cr; k++)
 			{
-				for (k=cl+1; k<cr; k++)
-				{
-					if (picture[row][k] != ' ') break;
-				}
-				if (k == cr) break;
+				if (picture[row][k] != ' ') break;
 			}
+			if (k == cr) break;
+		}
 
-			if (NULL != pctx) /* PS junk */
-			{
-				/* We know it fits, so put it in this row */
-				pctx->link_heights[j] = row;
+		if (NULL != pctx) /* PS junk */
+		{
+			/* We know it fits, so put it in this row */
+			pctx->link_heights[j] = row;
+		}
+
+		if (2*row+2 > max_height-1) {
+			lgdebug(+9, "Extending rows up to %u.\n", (2*row+2)+HEIGHT_INC);
+			diagram_alloc_tmpmem(&start, &picture, &xpicture,
+			                     &max_height, (2*row+2)+HEIGHT_INC,
+			                     max_bytes, num_cols);
+		}
+		if (row > top_row) top_row = row;
+
+		for (k=cl+1; k<cr; k++) {
+			picture[row][k] = '-';
+		}
+
+		s = lnk->link_name;
+		k = strlen(s);
+		inc = cl + cr + 2;
+		if (inc < k) inc = 0;
+		else inc = (inc-k)/2;
+		if (inc <= cl) {
+			t = picture[row] + cl + 1;
+		} else {
+			t = picture[row] + inc;
+		}
+
+		/* Add direction indicator */
+		if (DEPT_CHR == connector_string(lnk->lc)[0] &&
+		    (t > &picture[row][cl])) { picture[row][cl+1] = '<'; }
+		if (HEAD_CHR == connector_string(lnk->lc)[0]) { *(t-1) = '>'; }
+
+		/* Copy connector name; stop short if no room */
+		while ((*s != '\0') && (*t == '-')) *t++ = *s++;
+
+		/* Add direction indicator */
+		if (DEPT_CHR == connector_string(lnk->rc)[0]) { picture[row][cr-1] = '>'; }
+		if (HEAD_CHR == connector_string(lnk->rc)[0]) { *t = '<'; }
+
+		picture[row][cl] = '+';
+		picture[row][cr] = '+';
+
+		/* Now put in the | below this one, where needed */
+		for (k=0; k<row; k++) {
+			if (picture[k][cl] == ' ') {
+				picture[k][cl] = '|';
 			}
-
-			if (2*row+2 > max_height-1) {
-				lgdebug(+9, "Extending rows up to %d.\n", (2*row+2)+HEIGHT_INC);
-				diagram_alloc_tmpmem(&start, &picture, &xpicture,
-				                     &max_height, (2*row+2)+HEIGHT_INC,
-				                     max_bytes, num_cols);
-			}
-			if (row > top_row) top_row = row;
-
-			picture[row][cl] = '+';
-			picture[row][cr] = '+';
-			for (k=cl+1; k<cr; k++) {
-				picture[row][k] = '-';
-			}
-
-			s = ppla[j].link_name;
-			k = strlen(s);
-			inc = cl + cr + 2;
-			if (inc < k) inc = 0;
-			else inc = (inc-k)/2;
-			if (inc <= cl) {
-				t = picture[row] + cl + 1;
-			} else {
-				t = picture[row] + inc;
-			}
-
-			/* Add direction indicator */
-			// if (DEPT_CHR == ppla[j]->lc->string[0]) { *(t-1) = '<'; }
-			if (DEPT_CHR == connector_string(ppla[j].lc)[0] &&
-			    (t > &picture[row][cl])) { picture[row][cl+1] = '<'; }
-			if (HEAD_CHR == connector_string(ppla[j].lc)[0]) { *(t-1) = '>'; }
-
-			/* Copy connector name; stop short if no room */
-			while ((*s != '\0') && (*t == '-')) *t++ = *s++;
-
-			/* Add direction indicator */
-			// if (DEPT_CHR == ppla[j]->rc->string[0]) { *t = '>'; }
-			if (DEPT_CHR == connector_string(ppla[j].rc)[0]) { picture[row][cr-1] = '>'; }
-			if (HEAD_CHR == connector_string(ppla[j].rc)[0]) { *t = '<'; }
-
-			/* The direction indicators may have clobbered these. */
-			picture[row][cl] = '+';
-			picture[row][cr] = '+';
-
-			/* Now put in the | below this one, where needed */
-			for (k=0; k<row; k++) {
-				if (picture[k][cl] == ' ') {
-					picture[k][cl] = '|';
-				}
-				if (picture[k][cr] == ' ') {
-					picture[k][cr] = '|';
-				}
+			if (picture[k][cr] == ' ') {
+				picture[k][cr] = '|';
 			}
 		}
 	}
@@ -831,11 +807,6 @@ void linkage_free_links_and_domains(char * s)
 	exfree(s, strlen(s)+1);
 }
 
-void linkage_free_senses(char * s)
-{
-	exfree(s, strlen(s)+1);
-}
-
 /* Forward declarations, the gunk is at the bottom. */
 static const char * trailer(bool print_ps_header);
 static const char * header(bool print_ps_header);
@@ -1278,24 +1249,24 @@ static const char * header(bool print_ps_header)
 /**
  * Print elements of the 2D-word-array produced for the parsers.
  *
- * - print_sentence_word_alternatives(s, sent, false, NULL, tokenpos)
+ * - print_sentence_word_alternatives(s, sent, false, NULL, NULL, tokenpos)
  * If a pointer to struct "tokenpos" is given, return through it the index of
  * the first occurrence in the sentence of the given token. This is used to
  * prevent duplicate information display for repeated morphemes (if there are
  * multiples splits, each of several morphemes, otherwise some of them may
  * repeat).
  *
- * - print_sentence_word_alternatives(s, sent, true, NULL, NULL)
+ * - print_sentence_word_alternatives(s, sent, true, NULL, NULL, NULL)
  * If debugprint is "true", this is a debug printout of the sentence.  (The
  * debug printouts are with level 0 because this function is invoked for debug
  * on certain positive level.)
  *
  *
- * - print_sentence_word_alternatives(s, sent, false, display_func, NULL)
+ * - print_sentence_word_alternatives(s, sent, false, display_func, arg, NULL)
  * Iterate over the sentence words and their alternatives.  Handle each
- * alternative using the display_func function if it is supplied, or else (if it
- * is NULL) just print them. It is used to display disjunct information when
- * command !!word is used.
+ * alternative using display_func(..., arg) if it is supplied, or else (if
+ * display_func is NULL) just print them. It is used to display disjunct
+ * information when command !!word is used.
  * FIXME In the current version (using Wordgraph) the "alternatives" in the
  * word-array don't necessarily consist of real word alternatives.
  *
@@ -1308,8 +1279,10 @@ struct tokenpos /* First position of the given token - to prevent duplicates */
 	size_t ai;
 };
 
-void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint,
-     char * (*display)(Dictionary, const char *), struct tokenpos * tokenpos)
+void print_sentence_word_alternatives(dyn_str *s, Sentence sent,
+        bool debugprint,
+        char * (*display)(Dictionary, const char *, const void **),
+        const void **arg, struct tokenpos *tokenpos)
 {
 	size_t wi;   /* Internal sentence word index */
 	size_t ai;   /* Index of a word alternative */
@@ -1332,11 +1305,12 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 	{
 		/* For analyzing words we need to ignore the left/right walls */
 		if (dict->left_wall_defined &&
-		    (0 == strcmp(sent->word[0].unsplit_word, LEFT_WALL_WORD)))
+		    ((NULL != sent->word[0].alternatives[0])) &&
+		    (0 == strcmp(sent->word[0].alternatives[0], LEFT_WALL_WORD)))
 			first_sentence_word = 1;
 		if (dict->right_wall_defined &&
-		    ((NULL != sent->word[sentlen-1].unsplit_word)) &&
-		    (0 == strcmp(sent->word[sentlen-1].unsplit_word, RIGHT_WALL_WORD)))
+		    ((NULL != sent->word[sentlen-1].alternatives[0])) &&
+		    (0 == strcmp(sent->word[sentlen-1].alternatives[0], RIGHT_WALL_WORD)))
 			sentlen--;
 
 		/* Find if a word got split. This is indicated by:
@@ -1364,7 +1338,7 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 				}
 			}
 		}
-		/* "String", because it can be a word, morpheme, or (TODO) idiom */
+		/* "String", because it can be a word, morpheme, or idiom */
 		if (word_split && (NULL == display)) dyn_strcat(s, "String splits to:\n");
 		/* We used to print the alternatives of the word here, one per line.
 		 * In the current (Wordgraph) version, the alternatives may look
@@ -1451,7 +1425,7 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 				{
 					struct tokenpos firstpos = { wt };
 
-					print_sentence_word_alternatives(s, sent, false, NULL, &firstpos);
+					print_sentence_word_alternatives(s, sent, false, NULL, NULL, &firstpos);
 					if (((firstpos.wi != wi) || (firstpos.ai != ai)) &&
 					  firstpos.wi >= first_sentence_word) // allow !!LEFT_WORD
 					{
@@ -1482,7 +1456,7 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 				 * Display the features of the token. */
 				if ((NULL == tokenpos) && (NULL != display))
 				{
-					char *info = display(sent->dict, wt);
+					char *info = display(sent->dict, wt, arg);
 
 					if (NULL == info) return;
 					append_string(s, "Token \"%s\" ", wt);
